//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LightGreen
#property indicator_color2  Thistle
#property indicator_color3  DimGray
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_level1  0.0
#property indicator_levelcolor DimGray

//
//
//
//
//

extern int    RepulseLength    = 5;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = true;
extern bool   alertsEmail      = false;
extern bool   alertsNotify     = false;
extern string soundFile        = "alert2.wav";
extern bool   ShowArrows       = true;
extern string arrowsIdentifier = "repulse Arrows1";
extern double arrowsUpperGap   = 1.0;
extern double arrowsLowerGap   = 1.0;
extern color  arrowsUpColor    = LimeGreen;
extern color  arrowsDnColor    = Red;
extern int    arrowsUpCode     = 241;
extern int    arrowsDnCode     = 242;

//
//
//
//
//

double repulse[];
double repulsh[];
double repulsl[];
double buffa[];
double buffb[];
double trend[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0, repulsh); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, repulsl); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2, repulse); 
   SetIndexBuffer(3, buffa); 
   SetIndexBuffer(4, buffb); 
   SetIndexBuffer(5, trend);
   return(0);
}

int deinit() 
{  
   deleteArrows(); 
   
return(0); 
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //

   double alpha = 2.0/(1.0+5.0*RepulseLength);
   for(i=limit; i>=0; i--)
   {
      double pricea = 100*(3.0*Close[i]-2.0*Low[iLowest(NULL,0,MODE_LOW,RepulseLength,i)]-Open[i+RepulseLength])/Close[i];
      double priceb = 100*(Open[i+RepulseLength]+2.0*High[iHighest(NULL,0,MODE_HIGH,RepulseLength,i)]-3.0*Close[i])/Close[i];
      if (i==Bars-1)
      {
         buffa[i] = pricea;
         buffb[i] = priceb;
         continue;
      }
      buffa[i] = buffa[i+1]+alpha*(pricea-buffa[i+1]);
      buffb[i] = buffb[i+1]+alpha*(priceb-buffb[i+1]);

         repulse[i] = buffa[i]-buffb[i];
         repulsh[i] = EMPTY_VALUE;
         repulsl[i] = EMPTY_VALUE;
         trend[i] = trend[i+1];
         if (repulse[i]>0)  trend[i] = 1;
         if (repulse[i]<0)  trend[i] =-1;
         if (trend[i] == 1) repulsh[i] = repulse[i];
         if (trend[i] ==-1) repulsl[i] = repulse[i]; 
         
         //
         //
         //
         //
         //
         
         if (ShowArrows)
         {
           deleteArrow(Time[i]);
           if (trend[i] != trend[i+1])
           {
             if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
             if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
           }
         }
   }
   
   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert(" crossed zero up");
      else  doAlert(" crossed zero down");       
    }         
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Repulse 2 ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Repulse 2 "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}


